% Constructor of the finite elements block class. This is the base class 
% from which more specialized (heat diffusion, elasticity,...) finite 
% element block classes are derived.
%
% function retobj = feblock (varargin)
%
% See discussion of constructors in <a href="matlab:helpwin 'FAESOR/classes/Contents'">classes/Contents</a>. 
% Parameters:
%  mandatory:
%      gcells = cell array of geometry cells,
%  optional:
%      mater =material,
%      integration_rule= integration rule object
%      Rm= transformation matrix, or a handle to function to compute the 
%   transformation matrix, in columns are the basis vectors expressed in
%   the global Cartesian basis. If the transformation matrix is not supplied, 
%   geniso_Rm() is assumed. The transformation matrix may be passed in as
%   empty ([]), and the finite element block can take advantage of that by
%   assuming that Rm being empty means the local to global transformation
%   is the identity and avoiding thereby the multiplication.
%
function retobj = feblock (varargin)
    class_name='feblock';
    parent=classbase;
    if (nargin == 0)
        self.mater = [];
        self.gcells = [];
        self.integration_rule = [];
        self.Rm=[];
        retobj = class(self,class_name,parent);
        return;
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        else
            Parameters =varargin{1};
            self.mater = [];
            self.gcells = [];
            self.integration_rule = [];
            self.Rm=[];
            if isfield(Parameters,'mater')
                self.mater = Parameters.mater;
            end
            self.gcells = Parameters.gcells;
            if (count(self.gcells) < 1)
                error('Need at least one gcell!');
            end
            if isfield(Parameters,'integration_rule')
                self.integration_rule = Parameters.integration_rule;
            end
            Rm=[];
            if isfield(Parameters,'Rm')
                Rm= Parameters.Rm;
                if (~isempty(Rm)) && strcmp(class(Rm),'double')
                    if norm(Rm'*Rm-eye(size(Rm, 2)))>1e-9
                        error('Non-orthogonal local basis matrix!');
                    end
                elseif strcmp(class(Rm),'function_handle')
                elseif ((~isempty(Rm)))
                    error(['Cannot handle class of Rm: ' class(Rm)])
                end
            end
%             if (isempty(Rm))
%                 Rm=@geniso_Rm;
%             end
            self.Rm=Rm;
            retobj = class(self,class_name,parent);
            return;
        end
    else
        error('Illegal arguments!');
    end
    return;
end

