% Compute the element load vectors corresponding to prescribed pressure
% flux. (Pressure flux boundary condition is stated as
%      rho*a_n=-(d p/d n)
% where a_n= prescribed acceleration, n=direction along the outer normal
%
% function evs = pressure_flux_loads (self, geom, P, fi)
%
% Return an array of the element loads so they may be assembled.
%     
% Arguments
%     geom=geometry field
%     P=pressure field
%     Force = field for the force components
%
function ems = pressure_to_resultant_force (self, geom, P, Force)
    gcells = get(self.feblock,'gcells');
    nfens = get(gcells,'nfens');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    % Precompute basis f. values + basis f. gradients wrt parametric coor
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    % Prepare some data: connectivity and the element matrices
    conns = get(gcells, 'conn'); % connectivity
    xs =get(geom,'values');% Retrieve Geometry information
    Ge = cell(size(conns,1),1);
    eqnums_row = cell(size(conns,1),1);
    eqnums_col = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums_row{i} =gather(Force,1:3,'eqnums');
        eqnums_col{i} =gather(P,conns(i,:),'eqnums');
        Ge{i} =zeros(3,nfens);
    end
    xs =get(geom,'values');
    for i=1:size(conns,1)
        conn =conns(i,:);
        x=xs(conn,:);
        for j=1:npts_per_gcell
            Y=Ns{j}'*x; % Point at which pressure force acts
            J = Jacobian_matrix(gcells,Nders{j},x);
            Jac = Jacobian_surface(gcells,conn, Ns{j}, J, x);
            n= normal(self,Y,J);
            Ge{i} = Ge{i} + n*Ns{j}'*(Jac*w(j));
        end
    end
    ems  = elematset(struct('mat',{Ge}, 'eqnums_row',{eqnums_row},'eqnums_col',{eqnums_col}));
end
