% Compute the geometric stiffness matrices of the individual beam gcells.
% Return an array of them so they may be assembled.
%    
% function ems = stiffness_geo (self, geom0, u1, Rfield1, dchi)
% 
%     geom0=initial geometry field; locations of nodes
%     u1=current geometry increment field; displacements of nodes
%     Rfield1=field of nodal frames, current deformation
%     dchi=field of translation and rotation increments
%             Equation numbers are taken from the field dchi.   
%
% The stiffness matrices are computed in the local coordinate system and
% then transformed into the global coordinate system. 
% Equation numbers are taken from the field dchi. 
% 
% This geometric stiffness is based on the consistent linearization of the
% corotational expression for the internal (resisting) force. See Krenk's
% 2009 book.
function ems = stiffness_geo (self, geom0, u1, dchi)
    gcells = get(self.feblock,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(geom0,'dim');
    conns = get(gcells, 'conn'); % connectivity
    % Pre-allocate 
    Kg = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        e1=gather(dchi,conns(i,1),'eqnums');
        e2=gather(dchi,conns(i,2),'eqnums');
        eqnums{i} =[e1(1:3);e2(1:3)];
        Kg{i} =[];
    end
    Xs =gather(geom0,(1:get(geom0,'nfens')),'values','noreshape');
    U1s =gather(u1,(1:get(u1,'nfens')),'values','noreshape');
    X1s=Xs+U1s;
    % Now loop over all gcells in the block
    for i=1:ngcells
        conn = conns(i,:); % connectivity
        x0 = Xs(conn,:); % initial coordinates of nodes
        x1 = X1s(conn,1:3); % current coordinates of nodes
        [L0,L1,Ft]=local_frames(x0,x1);
        N=self.Force_function(L0,L1);
        if (L1 >0)
            s11=[0,0,0;
                0,N/L1,0;
                0,0,N/L1];
            K11=Ft*s11*Ft';
        else
            K11=zeros(3);
        end
        Kg{i} = [K11,-K11;-K11,K11];% 
    end
    ems  = elematset(struct('mat',{Kg}, 'eqnums',{eqnums}));
end
% syms N L real
% s11=[0,0,0;
%             0,N/L,0;
%             0,0,N/L];
% s12=[0,0,0;
%             0,-N/L,0;
%             0,0,-N/L]; s21=s12';
%         SM=[s11,s12;
%             s21,s11;
%             ];
%         syms T11 T12 T13 real
%         syms T21 T22 T23 real
%         syms T31 T32 T33 real
%         Te1=[T11 T12 T13
%              T21 T22 T23
%              T31 T32 T33];
%          Te= [Te1,0*Te1;0*Te1,Te1];
%          Te1*s11*Te1'
%          Te*SM*Te'
%         Be= [F1t',F1t'];%In global coordinate system
%         Kg{i} = Te*Kg{i}*Te';% transform to global coordinates 