% Compute the strain-displacement matrix for a three-manifold element.
%
% B = feblock_defor_ss_Blmat3(self,N,Ndersp,c,Rm)
%
% Compute the linear, displacement independent, strain-displacement matrix
% for a three-manifold element.
%    
%   Arguments
%      self=gcell
%      N= matrix of basis function values
%      Ndersp=matrix of basis function gradients
%      c=array of spatial coordinates of the evaluation point
%      Rm=orthogonal matrix with the unit basis vectors of the local
%         coordinate system as columns; supply Rm as empty when there 
%         is no need for global-to-local transformation (for instance for 
%         isotropic materials).
%         size(Rm)= [ndim,ndim], where ndim = Number of spatial dimensions
%         of the embedding space.
% Output: B = strain-displacement matrix, size (B) = [nstrain,nfens*dim],
%      where nstrain= number of strains, dim = Number of spatial dimensions of
%      the embedding space, and nfens = number of finite element nodes on
%      the element.
%
function B = feblock_defor_ss_Blmat3(self,N,Ndersp,c,Rm)
    nfn= size(Ndersp,1);
    B = zeros(6,nfn*3); %initialize
    if (isempty(Rm)) % there is no global-to-local transformation
        for i= 1:nfn
            k=3*(i-1);
            B(1,k+1)= Ndersp(i,1);
            B(2,k+2)= Ndersp(i,2);
            B(3,k+3)= Ndersp(i,3) ;
            B(4,k+1)= Ndersp(i,2); B(4,k+2)= Ndersp(i,1);
            B(5,k+1)= Ndersp(i,3); B(5,k+3)= Ndersp(i,1);
            B(6,k+2)= Ndersp(i,3); B(6,k+3)= Ndersp(i,2);
        end
    else % global-to-local transformation is requested
        RmT=Rm';
        for i= 1:nfn
            B(:,3*(i-1)+1:3*i)...
                = [ Ndersp(i,1) 0           0  ; ...
                0           Ndersp(i,2) 0  ; ...
                0           0           Ndersp(i,3) ; ...
                Ndersp(i,2) Ndersp(i,1) 0  ; ...
                Ndersp(i,3) 0           Ndersp(i,1) ; ...
                0           Ndersp(i,3) Ndersp(i,2) ]*RmT;
        end
    end
    return;
end


% 43.7 seconds
%     for i= 1:nfn
%         B(:,3*(i-1)+1:3*i)...
%             = [ Ndersp(i,1) 0           0  ; ...
%             0           Ndersp(i,2) 0  ; ...
%             0           0           Ndersp(i,3) ; ...
%             Ndersp(i,2) Ndersp(i,1) 0  ; ...
%             Ndersp(i,3) 0           Ndersp(i,1) ; ...
%             0           Ndersp(i,3) Ndersp(i,2) ]*Rm';
%     end

% 45 seconds
% dim=3;
%         Rm_1=(Rm(:,1))';
%     Rm_2=(Rm(:,2))';
%     Rm_3=(Rm(:,3))';
%     for i= 1:nfn
%         B(:,dim*(i-1)+1:dim*i) =[Ndersp(i,1)*Rm_1;...
%              Ndersp(i,2)*Rm_2;...
%              Ndersp(i,3)*Rm_3;...
%              Ndersp(i,2)*Rm_1 + Ndersp(i,1)*Rm_2;...
%              Ndersp(i,3)*Rm_1 + Ndersp(i,1)*Rm_3;...
%              Ndersp(i,3)*Rm_2 + Ndersp(i,2)*Rm_3];
%     end

% % 45.5 seconds
% Rm_1=(Rm(:,1))';
% Rm_2=(Rm(:,2))';
% Rm_3=(Rm(:,3))';
% for i= 1:nfn
%     k=3*(i-1); k1 =k+1:k+3;
%     B(1,k1)= Ndersp(i,1)*Rm_1;
%     B(2,k1)= Ndersp(i,2)*Rm_2;
%     B(3,k1)= Ndersp(i,3)*Rm_3;
%     B(4,k1)= Ndersp(i,2)*Rm_1 + Ndersp(i,1)*Rm_2;
%     B(5,k1)= Ndersp(i,3)*Rm_1 + Ndersp(i,1)*Rm_3;
%     B(6,k1)= Ndersp(i,3)*Rm_2 + Ndersp(i,2)*Rm_3;
% end

% 34 seconds
%    for i= 1:nfn
%         k=3*(i-1);
%         B(1,k+1)= Ndersp(i,1);
%         B(2,k+2)= Ndersp(i,2);
%         B(3,k+3)= Ndersp(i,3) ;
%         B(4,k+1)= Ndersp(i,2); B(4,k+2)= Ndersp(i,1);
%         B(5,k+1)= Ndersp(i,3); B(5,k+3)= Ndersp(i,1);
%         B(6,k+2)= Ndersp(i,3); B(6,k+3)= Ndersp(i,2);
%     end

% 37.4 seconds
% B(1,1:3:end) =Ndersp(:,1)';
% B(2,2:3:end) =Ndersp(:,2)';
% B(3,3:3:end) =Ndersp(:,3)';
% B(4,1:3:end) =Ndersp(:,2)';
% B(4,2:3:end) =Ndersp(:,1)';
% B(5,1:3:end) =Ndersp(:,3)';
% B(5,3:3:end) =Ndersp(:,1)';
% B(6,2:3:end) =Ndersp(:,3)';
% B(6,3:3:end) =Ndersp(:,2)';

