% Compute the local geometric stiffness matrix. 
%
% function SM = local_geometric_stiffness(self,A,I2,I3,PN,L)
% 
% Inputs:
% A= cross-sectional area, 
% I2, I3=central moment of inertia of the cross-section about the x2 and x3 
% coordinate axis, 
% PN= vector of natural forces; see natural_forces() for definitions
% L= current length of the element, 
% 
% Outputs:
% SM = local geometric stiffness matrix, 12 x 12
% 
% This geometric stiffness matrix this consistent with relationship between
% the natural deformations and the natural forces that assumes there is
% only a linear constitutive link: no non-constitutive effects (bowing
% etc.) are included. This form of the geometric matrix was derived by
% Krenk.
% @BOOK{Krenk:2009,
%   AUTHOR =       {S. Krenk},
%   TITLE =        {Non-linear Modeling and Analysis of Solids and Structures },
%   PUBLISHER =    {Cambridge University Press},
%   YEAR =         {2009},
%   isbn =         {9780521830546}
% } 
function SM = local_geometric_stiffness(self,A,I2,I3,PN,L)
    N=PN(1);
    S_2=-2*PN(3)/L;
    S_3=-2*PN(5)/L;
    M_1=PN(6);
    M_2I=PN(4)+PN(5);
    M_2J=PN(4)-PN(5);
    M_3I=-(PN(2)+PN(3));
    M_3J=-(PN(2)-PN(3));
    s11=[0,-S_2/L,-S_3/L;
        -S_2/L,N/L,0;
        -S_3/L,0,N/L];
    s12=[0,0,0;
        -M_2I/L,M_1/L,0;
        -M_3I/L,0,M_1/L]; s21=s12';
    s13=[0,+S_2/L,+S_3/L;
        +S_2/L,-N/L,0;
        +S_3/L,0,-N/L]; s31=s13';
    s14=[0,0,0;
        M_2J/L,-M_1/L,0;
        M_3J/L,0,-M_1/L]; s41=s14';
    s22=[0,M_3I/2,-M_2I/2;
        M_3I/2,0,0;
        -M_2I/2,0,0];
    s23=[0,M_2I/L,M_3I/L;
        0,-M_1/L,0;
        0,0,-M_1/L]; s32=s23';
    s24=[0,0,0;
        0,0,M_1/2;
        0,-M_1/2,0]; s42=s24';
    s33=[0,-S_2/L,-S_3/L;
        -S_2/L,N/L,0;
        -S_3/L,0,N/L];
    s34=[0,0,0;
        -M_2J/L,M_1/L,0;
        -M_3J/L,0,M_1/L]; s43=s34';
    s44=[0,-M_3J/2,M_2J/2;
        -M_3J/2,0,0;
        M_2J/2,0,0];
    SM=[s11,s12,s13,s14;
        s21,s22,s23,s24;
        s31,s32,s33,s34;
        s41,s42,s43,s44;
        ];
end
