% Compute the  thermal capacity matrices of the individual gcells.
%
% function ems = capacity(self, geom, temp)
%
% Return an array of them so they may be assembled.
%    Call as
%       ems = capacity(feb, geom, temp)
%    where
%       geom=geometry field
%       temp=temperature field
%
% 
% The specific heat may depend on the temperature.  The thermal
% property would define the specific heat as a function of the
% temperature, which is then evaluated at each quadrature point.
% 
function ems = capacity(self, geom, temp)
    gcells = get(self.feblock_diffusion,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells,'nfens');
    % Integration rule
    integration_rule = get(self.feblock_diffusion, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    % Material
    mat = get(self.feblock_diffusion, 'mater');
    cp = get(get(mat,'property'),'specific_heat');
    if (strcmp(class(cp),'function_handle'))
        need_to_eval=1;
    else
        need_to_eval=0; CP = cp;
    end
    conns = get(gcells, 'conn'); % connectivity
    Ce = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(temp,conns(i,:),'eqnums');
        Ce{i} =zeros(nfens);
    end
    xs =get(geom,'values');
    Ts = get(temp,'values');% node temperatures
    % Now loop over all gcells in the block
    for i=1:ngcells
        conn =conns(i,:);
        x=xs(conn,:);
        T=Ts(conn);
        for j=1:npts_per_gcell % Loop over integration points
            J = x' * Nders{j};%J = Jacobian_matrix(gcells,Nders{j},x);%J = x' * Nder;
            Jac = Jacobian_volume(gcells,conn, Ns{j}, J, x);
            if (need_to_eval)
                CP = cp (Ns{j}'*T);
            end
            Ce{i} = Ce{i} + Ns{j}*(Ns{j}' * (CP*Jac * w(j)));
        end
    end
    ems  = elematset(struct('mat',{Ce}, 'eqnums',{eqnums}));
end
