%%  Argyris L-frame, out-of-plane load

%% 
% References
% J.H. Argyris, P.C. Dunne and D.W. Scharpf, On large 
% displacement-small strain analysis of structures with 
% rotation degree of freedom. Comput. Methods Appl. Mech. Engrg. 14 (1978) 401-451; 15 (1978) 99-135. 
% J.H. Argyris, 0. Hilpert, GA. Malejannakis and D.W. Scharpf, On the 
% geometrical stiffness of a beam in space-A consistent V.W. 
% Approach, Comput. Methods Appl. Mech. Engrg. 20 (1979) 105-131. 
% 

%% 
% Verified with continuum element solution with high order hexahedron (Argyris_L_linearbuckling)
% Displacement of the point of application of the load in the direction of the force
%  0.0047, 0.0 in the remaining two directions

%% 
% Parameters:
E=71240.0;%MPa
nu=0.31;% Poisson ratio
b=0.6; h=30; L=240;% cross-sectional dimensions and length of each leg in millimeters
magn=1e-5;% Magnitude of the total applied force, Newton

%% 
% Mesh
n=8;
tolerance=L/n/100;
Parameters=struct ('A',b*h,'I2',b*h^3/12,'I3',b^3*h/12,'J',b^3*h/3,...
    'x1x2_vector',[0 1 0], 'label',1);
[fens1,gcells1] = Beam_member([0,0,L;L,0,L],n,@gcellset_beam3,Parameters);
[fens2,gcells2] = Beam_member([L,0,L;L,0,0],n,@gcellset_beam3,Parameters);
[fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens2, gcells2, tolerance);
gcells= cat(gcells1,gcells2);

% The clamped end is
clampedn=fenode_select(fens, struct('box', update_box([],[0,0,L]), 'inflate', tolerance));
% The force is applied at the tip
tipn=fenode_select(fens, struct('box', update_box([],[L,0,0]), 'inflate', tolerance));

%% 
% Properties
prop = property_linel_iso (struct('E',E,'nu',nu));
mater = mater_defor_ss_linel_triax (struct('property',prop));

%% 
% Finite element block
feb = feblock_defor_ss_beam3 (struct ('mater',mater, 'gcells',gcells));

%% 
% Geometry and displacement fields
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
ur  = field(struct ('name',['ur'], 'dim', 6, 'data', zeros(get(geom,'nfens'),6)));


%% 
% Apply EBC's
ebc_fenids=[clampedn];
ebc_prescribed=[1];
ebc_comp=[];
ebc_val=ebc_fenids*0;
ur  = set_ebc(ur, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ur  = apply_ebc (ur);


%% 
% Number equations
ur  = numbereqns (ur);
neqns=get(ur, 'neqns');


%% 
% Assemble and solve the discrete system
K = start (sparse_sysmat, neqns);
K = assemble (K, stiffness(feb, geom, ur));
% Load
F = sysvec;
F = start (F, neqns);
evs = loads(nodal_load(struct ('id',tipn,'dir',[2],'magn',[magn])), ur);
F = assemble (F, evs);
% Solve
ur = scatter_sysvec(ur, K\F);
load_displacement=somel(gather(ur,tipn,'values','noreshape'),1:3)


%% 
% Graphics display
gv=graphic_viewer;
gv=reset (gv,[]);
scale=2e4;
u=slice(ur,(1:3),'u');
rot=slice(ur,(4:6),'rot');
draw(feb, gv, struct ('x', geom, 'u', 0*u, 'rot', 0*rot, 'facecolor','none'));
draw(feb, gv, struct ('x', geom, 'u', scale*u, 'rot', scale*rot, 'facecolor','red'));

xlabel('X');
ylabel('Y');
zlabel('Z');
hold on

draw(fens,gv, struct ('x', geom, 'u', 0*u, 'color','blue','offset',0.35));
draw_axes(gv, struct([]));

assignin('caller','faesor_test_passed',(norm(load_displacement- [0    0.0047         0])<1e-3));
