%%  Argyris L-frame, out-of-plane load, nonlinear.
% 
% Out of plane load that produces a large deflection of the frame. Solved
% with a static nonlinear procedure.

%% 
% Parameters:
E=71240.0;%MPa
nu=0.31;% Poisson ratio
b=3; h=30; L=240;% cross-sectional dimensions and length of each leg in millimeters
magn=40;% Magnitude of the total applied force, Newton
utol=1e-3;

    
%% 
% Mesh
n=8;
tolerance=L/n/100;
Parameters=Beam_cross_section(struct('type','rectangle','d2',b,'d3',h));
Parameters.x1x2_vector=[0 1 0];
[fens1,gcells1] = Beam_member([0,0,L;L,0,L],n,@gcellset_beam3,Parameters);
[fens2,gcells2] = Beam_member([L,0,L;L,0,0],n,@gcellset_beam3,Parameters);
[fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens2, gcells2, tolerance);
gcells= cat(gcells1,gcells2);

% The clamped end is
clampedn=fenode_select(fens, struct('box', update_box([],[0,0,L]), 'inflate', tolerance));
% The force is applied at the tip
tipn=fenode_select(fens, struct('box', update_box([],[L,0,0]), 'inflate', tolerance));

%% 
% Properties
prop = property_linel_iso (struct('E',E,'nu',nu));
mater = mater_defor_ss_linel_triax (struct('property',prop));

%% 
% Finite element block
feb = feblock_defor_ss_corot_beam(struct ('mater',mater, 'gcells',gcells));

%% 
% Initialize configuration variables
geom0 = field(struct ('name',['geom0'], 'dim', 3, 'fens',fens));
u0=0*geom0;
Rfield = field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom0,'nfens'),1)*reshape(eye(3),1,9)));
Rfield0=Rfield;
% Define the displacement increment field
dchi  = field(struct ('name',['dchi'], 'dim', 6, 'data', zeros(get(geom0,'nfens'),6)));

% Apply EBC's
ebc_fenids=[clampedn];
ebc_prescribed=[1];
ebc_comp=[];
ebc_val=0;
dchi = set_ebc(dchi, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
dchi = apply_ebc (dchi);
% Number equations
dchi = numbereqns (dchi);
neqns=get(dchi, 'neqns');



gv=graphic_viewer;
    

load_parameters=linspace(0, 2, 10);
tip_displacement=[];
for load_parameter=load_parameters
    
    % Initialization
    dchi = 0*dchi; % guess
    dchi = apply_ebc(dchi);
    u1 = u0;
    Rfield1=Rfield;
    nl=nodal_load(struct ('id',tipn,'dir',[2],'magn',[-load_parameter*magn]));
    
    iter=1;
    while 1
        feb1=feb;  % Make a copy of the state
        F = start (sysvec, neqns);
        F = assemble (F, loads(nl, dchi));
        [feb1,evs] = restoring_force(feb1,geom0,u1,Rfield1,dchi);       % Internal forces
        F = assemble (F, evs);
        K = start (sparse_sysmat, neqns);
        % Iteration with the full stiffness matrix (including the geometric stiffness) is not going well;
        % an alternative is to iterate just with the elastic stiffness
        %                 K = assemble (K, cat(2,stiffness(feb1,geom0,u1,Rfield1,dchi),stiffness_geo(feb1,geom0,u1,Rfield1,dchi))); 
        K = assemble (K, cat(2,stiffness(feb1,geom0,u1,Rfield1,dchi)));
        dchi = scatter_sysvec(dchi, K\F); % Displacement increment
        % Line search
        R0 = dot(get(F,'vec'),gather_sysvec(dchi));
        F = start (sysvec, neqns);
        F = assemble (F, loads(nl, dchi));
        [feb1,evs] = restoring_force(feb1,geom0,...
            u1+slice(dchi,(1:3),'transl'),update_rotation_field(Rfield1,slice(dchi,(4:6),'rotat')),dchi);       % Internal forces
        F = assemble (F, evs);
        R1 = dot(get(F,'vec'),gather_sysvec(dchi));
        a = R0/R1;
        if ( a<0 )
            eta = a/2 +sqrt((a/2)^2 -a);
        else
            eta =a/2;
        end
        eta=min( [eta, 1.0] );% line search concluded
        %         eta=1; 
        u1 = u1 + eta*slice(dchi,(1:3),'transl');   % increment displacement
        Rfield1=update_rotation_field(Rfield1,eta*slice(dchi,(4:6),'rotat'));
        draw(feb, gv, struct ('x', geom0, 'u', u1, 'R', Rfield1, 'facecolor','g'));
        disp(['   It. ' num2str(iter) ': ||du||=' num2str(norm(dchi))]);
        if (max(abs(gather(dchi,[],'values'))) < utol) break; end;                    % convergence check
        iter=iter+1;
    end
    [feb,ignore] = restoring_force(feb,geom0,u1,Rfield1,dchi);        % final update
    disp(['    Converged for load_parameter=' num2str(load_parameter)]); % pause
    u0 = u1; % update the displacement
    Rfield0=Rfield1; % update the rotations
    utip=gather(u1,get(nl, 'id'),'values');
    disp(['Tip deflection =' num2str(utip(1:3)','%27.15e, %27.15e, %27.15e\n')])
    tip_displacement=[tip_displacement;utip(1:3)'];
    
    %%
    % Graphic display of the mesh and the deformed beam
    gv=reset (gv,[]);
    scale=1;
    draw(feb, gv, struct ('x', geom0, 'u', [], 'R', [], 'facecolor','none'));
    draw(feb, gv, struct ('x', geom0, 'u', u1, 'R', Rfield1, 'facecolor','g'));
%     draw_local_coordinates(feb, gv, struct ('x', geom, 'u', scale*u, 'rot', 0*rot, 'length',25));
    
    xlabel('X');
    ylabel('Y');
    zlabel('Z');
    hold on
    
    %     draw(fens,gv, struct ('x', geom, 'u', 0*u, 'color','blue','offset',0.35)); 
    draw_axes(gv, struct([]));
    pause(1)
end
